Namespace Petzold.Astronomy
	Public MustInherit Class CelestialBody
		' Only used internally
		Private Property Location As GeographicCoordinate

		' Set here, used by descendent classes
        Private _time As Time
        Protected Property Time As Time
            Get
                Return _time
            End Get
            Private Set(ByVal value As Time)
                _time = value
            End Set
        End Property

		' Set by descendent classes, used here
        Private _equatorialCoordinate As EquatorialCoordinate
        Protected Property EquatorialCoordinate As EquatorialCoordinate
            Private Get
                Return _equatorialCoordinate
            End Get
            Set(ByVal value As EquatorialCoordinate)
                _equatorialCoordinate = value
            End Set
        End Property

		' Set here, used external to library
        Private _horizontalCoordinate As HorizontalCoordinate
        Public Property HorizontalCoordinate As HorizontalCoordinate
            Get
                Return _horizontalCoordinate
            End Get
            Private Set(ByVal value As HorizontalCoordinate)
                _horizontalCoordinate = value
            End Set
        End Property

		' Used externally to retain screen location
		Public ScreenX As Single
		Public ScreenY As Single

		' Called external to update HorizontalCoordinate
		Public Sub Update(ByVal time As Time, ByVal location As GeographicCoordinate)
			Dim needsUpdate As Boolean = False

			If Not Me.Time.Equals(time) Then
				Me.Time = time
				needsUpdate = True
				OnTimeChanged()
			End If

			If Not Me.Location.Equals(location) Then
				Me.Location = location
				needsUpdate = True
			End If

			If needsUpdate Then
				Me.HorizontalCoordinate = HorizontalCoordinate.From(Me.EquatorialCoordinate, Me.Location, Me.Time)
			End If
		End Sub

		' Overridden by descendent classes to update EquatorialCoordinate
		Protected Overridable Sub OnTimeChanged()
		End Sub
	End Class
End Namespace
