Imports System.Xml.Serialization

Namespace Petzold.Astronomy
	Public Class Constellation
		Inherits CelestialBody

		Public Sub New()
			Stars = New List(Of Star)
			Connectors = New List(Of Connector)
		End Sub

		<XmlAttribute>
		Public Property Name As String

		<XmlAttribute>
		Public Property LongName As String

		<XmlElement(ElementName := "Star")>
		Public Property Stars As List(Of Star)

		<XmlElement(ElementName := "Connector")>
		Public Property Connectors As List(Of Connector)

		<XmlIgnore>
		Public Property StarConnectors As List(Of StarConnector)

		Public Sub Initialize()
			For Each connector As Connector In Me.Connectors
				FindStar(connector.From).IsConnectedInConstellation = True
				FindStar(connector.To).IsConnectedInConstellation = True
			Next connector

			' The following calculation is solely for positioning the name
			'  of the constellation relative to its contents.

			' First determine if the constellation crosses the 0° right ascension line
			Dim numLessThan8Hours As Integer = 0
			Dim numGreaterThan16Hours As Integer = 0

			For Each star As Star In Me.Stars
				If star.RightAscension < 8 Then
					numLessThan8Hours += 1
				ElseIf star.RightAscension > 16 Then
					numGreaterThan16Hours += 1
				End If
			Next star

			Dim straddles As Boolean = numLessThan8Hours > 0 AndAlso numGreaterThan16Hours > 0
			Dim countAll As Boolean = Me.Connectors.Count = 0

			Dim count As Integer = 0
			Dim rightAscensionTotal As Double = 0
			Dim declinationTotal As Double = 0

			For Each star As Star In Me.Stars
				If countAll OrElse star.IsConnectedInConstellation Then
					count += 1
					rightAscensionTotal += star.RightAscension - (If(straddles AndAlso star.RightAscension > 16, 24, 0))
					declinationTotal += star.Declination
				End If
			Next star

			Me.EquatorialCoordinate = New EquatorialCoordinate(Angle.FromHours(rightAscensionTotal / count), Angle.FromDegrees(declinationTotal / count))
		End Sub

		Public Function FindStar(ByVal number As Integer) As Star
			For Each star As Star In Me.Stars
				If star.Number = number Then
					Return star
				End If
			Next star

			Return Nothing
		End Function
	End Class
End Namespace
