Namespace Petzold.Astronomy
	Public Structure EquatorialCoordinate
		Public Sub New(ByVal rightAscension As Angle, ByVal declination As Angle)
            Me.RightAscension = rightAscension
			Me.Declination = declination
		End Sub

        Private _rightAscension As Angle
        Public Property RightAscension As Angle
            Get
                Return _rightAscension
            End Get
            Private Set(ByVal value As Angle)
                _rightAscension = value
            End Set
        End Property

        Private _declination As Angle
        Public Property Declination As Angle
            Get
                Return _declination
            End Get
            Private Set(ByVal value As Angle)
                _declination = value
            End Set
        End Property

		' Time not used in current calculation, but might be need in future refinements
		Public Shared Function From(ByVal eclipticCoordinate As EclipticCoordinate, ByVal time As Time) As EquatorialCoordinate
			' Calculate right ascension and declination
			Dim angleEpsilon As Angle = Angle.FromDegrees(23.4392911) ' obliquity of the ecliptic -- see Meeus, pg. 92

			Dim angleRightAscension As Angle = Angle.ArcTangent(eclipticCoordinate.Longitude.Sine * angleEpsilon.Cosine - eclipticCoordinate.Latitude.Tangent * angleEpsilon.Sine, eclipticCoordinate.Longitude.Cosine)

			angleRightAscension.NormalizePositive()

			Dim angleDeclination As Angle = Angle.Zero

			angleDeclination.Sine = eclipticCoordinate.Latitude.Sine * angleEpsilon.Cosine + eclipticCoordinate.Latitude.Cosine * angleEpsilon.Sine * eclipticCoordinate.Longitude.Sine

			angleDeclination.NormalizeAroundZero()

			Return New EquatorialCoordinate(angleRightAscension, angleDeclination)
		End Function
	End Structure
End Namespace
