Namespace Petzold.Astronomy
	Public Structure GeographicCoordinate
		Public Sub New(ByVal longitude As Angle, ByVal latitude As Angle)
            Me.Longitude = longitude
			Me.Latitude = latitude
		End Sub

        Private _longitude As Angle
        Public Property Longitude As Angle
            Get
                Return _longitude
            End Get
            Private Set(ByVal value As Angle)
                _longitude = value
            End Set
        End Property

        Private _latitude As Angle
        Public Property Latitude As Angle
            Get
                Return _latitude
            End Get
            Private Set(ByVal value As Angle)
                _latitude = value
            End Set
        End Property

		Public Shared ReadOnly Property NewYorkCity As GeographicCoordinate
			Get
				Return New GeographicCoordinate(Angle.FromDegrees(73.99), Angle.FromDegrees(40.73))
			End Get
		End Property

		' Subtraction operator returns distance along unit sphere.
		Public Shared Operator -(ByVal gc1 As GeographicCoordinate, ByVal gc2 As GeographicCoordinate) As Double
			Dim angBetween As Angle = Angle.Zero
			angBetween.Cosine = gc1.Latitude.Sine * gc2.Latitude.Sine + gc1.Latitude.Cosine * gc2.Latitude.Cosine * (gc1.Longitude - gc2.Longitude).Cosine
			Return angBetween.Radians
		End Operator

		' Operators and overrides
		Public Shared Operator =(ByVal geo1 As GeographicCoordinate, ByVal geo2 As GeographicCoordinate) As Boolean
			Return geo1.Equals(geo2)
		End Operator

		Public Shared Operator <>(ByVal geo1 As GeographicCoordinate, ByVal geo2 As GeographicCoordinate) As Boolean
			Return Not geo1.Equals(geo2)
		End Operator

		Public Overrides Overloads Function Equals(ByVal obj As Object) As Boolean
			If Not(TypeOf obj Is GeographicCoordinate) Then
				Return False
			End If

			Return Equals(CType(obj, Angle))
		End Function

		Public Overloads Function Equals(ByVal that As GeographicCoordinate) As Boolean
			Return Me.Longitude.Equals(that.Longitude) AndAlso Me.Latitude.Equals(that.Latitude)
		End Function

		Public Overrides Function GetHashCode() As Integer
			Dim longitude As Angle = Me.Longitude
			Dim latitude As Angle = Me.Latitude

			longitude.NormalizePositive()
			latitude.NormalizePositive()

			Return (180 * longitude + latitude).GetHashCode
		End Function

		Public Overrides Function ToString() As String
			Return String.Format("{0} longitude {1} latitude", Longitude, Latitude)
		End Function
	End Structure
End Namespace
