Imports System.IO
Imports System.Windows
Imports System.Windows.Resources

Namespace Petzold.Astronomy
	Public Class VsopCruncher
		' Structure used internally to store each term.
		Private Structure Term
			Public A, B, C As Double

			Public Function Calculate(ByVal tau As Double) As Double
				Return (A * Math.Cos(B + C * tau))
			End Function
		End Structure

		' termsArray basically stores all VSOP data.
        Private termsArray(2, 5)() As Term

		Public Sub New(ByVal strPlanetAbbreviation As String)
            Dim uri As New Uri("Data/VSOP87D." & strPlanetAbbreviation, UriKind.Relative)
			Dim resourceInfo As StreamResourceInfo = Application.GetResourceStream(uri)

			Using reader As New StreamReader(resourceInfo.Stream)
				Dim strLine As String

				strLine = reader.ReadLine
				Do While strLine IsNot Nothing
					' variable is 1, 2, or 3 corresponding to 
					'  L (ecliptical longitude), B (ecliptical latitude), or R (radius).
					Dim variable As Integer = Int32.Parse(strLine.Substring(41, 1))

					' terms is the number of lines that follow.
					Dim terms As Integer = Int32.Parse(strLine.Substring(60, 7))

					' power is the exponent of time.
					Dim power As Integer = Int32.Parse(strLine.Substring(59, 1))

					' Allocate a new array for all the terms that follow.
					Dim termArray(terms - 1) As Term

					' Fill up the termArray.
					For i  = 0 To terms - 1
						strLine = reader.ReadLine
						termArray(i).A = Double.Parse(strLine.Substring(79, 18))
						termArray(i).B = Double.Parse(strLine.Substring(97, 14))
						termArray(i).C = Double.Parse(strLine.Substring(111, 20))
					Next i

					' Save it as part of the big array.
					termsArray(variable - 1, power) = termArray
					strLine = reader.ReadLine
				Loop
			End Using
		End Sub

		Public Function GetLongitude(ByVal tau As Double) As Double
			Return Calculate(0, tau)
		End Function

		Public Function GetLatitude(ByVal tau As Double) As Double
			Return Calculate(1, tau)
		End Function

		Public Function GetRadius(ByVal tau As Double) As Double
			Return Calculate(2, tau)
		End Function

		Private Function Calculate(ByVal variable As Integer, ByVal tau As Double) As Double
			Dim final As Double = 0
			Dim tauPowered As Double = 1

			For power  = 0 To 5
				Dim termArray() As Term = termsArray(variable, power)

				If termArray IsNot Nothing Then
					Dim accum As Double = 0

					For term  = 0 To termArray.Length - 1
						accum += termArray(term).Calculate(tau)
					Next term

					final += accum * tauPowered
				End If
				tauPowered *= tau
			Next power
			Return final
		End Function
	End Class
End Namespace

